Package: qgraph
Type: Package
Title: Graph Plotting Methods, Psychometric Data Visualization and
        Graphical Model Estimation
Version: 1.9.8
Authors@R: c(
  person("Sacha", "Epskamp", email = "mail@sachaepskamp.com",role = c("aut", "cre")),
  person("Giulio", "Costantini", role = c("aut")),
  person("Jonas", "Haslbeck", role = c("aut")),
  person("Adela", "Isvoranu", role = c("aut")),
  person("Angelique O. J.", "Cramer", role = c("ctb")),
  person("Lourens J.", "Waldorp", role = c("ctb")),
  person("Verena D.", "Schmittmann", role = c("ctb")),
  person("Denny", "Borsboom", role = c("ctb")))
Maintainer: Sacha Epskamp <mail@sachaepskamp.com>
Depends: R (>= 3.0.0)
Imports: Rcpp (>= 1.0.0), methods, grDevices, psych, lavaan, plyr,
        Hmisc, igraph, jpeg, png, colorspace, Matrix, corpcor,
        reshape2, ggplot2, glasso, fdrtool, gtools, parallel, pbapply,
        abind
ByteCompile: yes
Description: Fork of qgraph - Weighted network visualization and analysis, as well as Gaussian graphical model computation. See Epskamp et al. (2012) <doi:10.18637/jss.v048.i04>.
BugReports: https://github.com/SachaEpskamp/qgraph
License: GPL-2
LazyLoad: yes
LinkingTo: Rcpp
Suggests: BDgraph, huge
NeedsCompilation: yes
Packaged: 2023-11-03 08:14:42 UTC; sachaepskamp
Author: Sacha Epskamp [aut, cre],
  Giulio Costantini [aut],
  Jonas Haslbeck [aut],
  Adela Isvoranu [aut],
  Angelique O. J. Cramer [ctb],
  Lourens J. Waldorp [ctb],
  Verena D. Schmittmann [ctb],
  Denny Borsboom [ctb]
Repository: CRAN
Date/Publication: 2023-11-03 11:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 13:33:00 UTC; unix
Archs: qgraph.so.dSYM
