This is the developmental version of qgraph. For the stable version please see the CRAN link to the package:
http://cran.r-project.org/web/packages/qgraph/index.html

For more information, please see the publication in Journal of Statistical Software:
http://www.jstatsoft.org/v48/i04/