\encoding{UTF8}
\name{big5}
\alias{big5}
\docType{data}
\title{
Big 5 dataset
}
\description{
This is a dataset of the Big 5 personality traits that will be used in talks and the paper. It is a measurement of the Dutch translation of the NEO-PI-R on 500 first year psychology students (Dolan, Oort, Stoel, Wicherts, 2009).
}
\usage{data(big5)}
\references{
Hoekstra, H. A., Ormel, J., & De Fruyt, F. (2003). NEO-PI-R/NEO-FFI: Big 5 persoonlijkheidsvragenlijst. Handleiding \[Manual of the Dutch version of the NEO-PI-R/NEO-FFI\]. Lisse, The Netherlands: Swets and Zeitlinger.

Dolan, C. V., Oort, F. J., Stoel, R. D., & Wicherts, J. M. (2009). Testing measurement invariance in the target rotates multigroup exploratory factor model. Structural Equation Modeling, 16, 295–314.
}
\format{
  The format is:
 num [1:500, 1:240] 2 3 4 4 5 2 2 1 4 2 ...
 - attr(*, "dimnames")=List of 2
  ..$ : NULL
  ..$ : chr [1:240] "N1" "E2" "O3" "A4" ...
}
